@@ -1,18 +1,21 @@
-use std::{collections::HashMap, convert::TryFrom, io::BufRead, result::Result};
+use std::{collections::HashMap, io::BufRead};
 
 use enumflags2::{bitflags, BitFlags};
 use serde::{Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 use static_assertions::assert_impl_all;
-use zbus::{dbus_proxy, fdo};
-use zvariant::{OwnedValue, Type, Value};
+use zbus::{
+    fdo,
+    names::OwnedUniqueName,
+    zvariant::{OwnedValue, Type, Value},
+};
 
 use crate::Error;
 
 /// Flags used in the CheckAuthorization() method.
 #[bitflags]
 #[repr(u32)]
-#[derive(Type, Debug, PartialEq, Copy, Clone, Serialize, Deserialize)]
+#[derive(Type, Debug, PartialEq, Eq, Copy, Clone, Serialize, Deserialize)]
 pub enum CheckAuthorizationFlags {
     /// If the Subject can obtain the authorization through authentication, and an authentication
     /// agent is available, then attempt to do so. Note, this means that the CheckAuthorization()
@@ -24,7 +27,7 @@ assert_impl_all!(CheckAuthorizationFlags: Send, Sync, Unpin);
 
 /// An enumeration for granting implicit authorizations.
 #[repr(u32)]
-#[derive(Deserialize_repr, Serialize_repr, Type, Debug, PartialEq)]
+#[derive(Deserialize_repr, Serialize_repr, Type, Debug, PartialEq, Eq)]
 pub enum ImplicitAuthorization {
     /// The Subject is not authorized.
     NotAuthorized = 0,
@@ -34,7 +37,8 @@ pub enum ImplicitAuthorization {
     AdministratorAuthenticationRequired = 2,
     /// Authentication is required. If the authorization is obtained, it is retained.
     AuthenticationRequiredRetained = 3,
-    /// Authentication as an administrator is required. If the authorization is obtained, it is retained.
+    /// Authentication as an administrator is required. If the authorization is obtained, it is
+    /// retained.
     AdministratorAuthenticationRequiredRetained = 4,
     /// The subject is authorized.
     Authorized = 5,
@@ -45,9 +49,10 @@ assert_impl_all!(ImplicitAuthorization: Send, Sync, Unpin);
 /// Flags describing features supported by the Authority implementation.
 #[bitflags]
 #[repr(u32)]
-#[derive(Type, Debug, PartialEq, Copy, Clone, Serialize, Deserialize)]
+#[derive(Type, Debug, PartialEq, Eq, Copy, Clone, Serialize, Deserialize)]
 pub enum AuthorityFeatures {
-    /// The authority supports temporary authorizations that can be obtained through authentication.
+    /// The authority supports temporary authorizations that can be obtained through
+    /// authentication.
     TemporaryAuthorization = 0x01,
 }
 
@@ -58,11 +63,12 @@ impl TryFrom<OwnedValue> for AuthorityFeatures {
 
     fn try_from(v: OwnedValue) -> Result<Self, Self::Error> {
         // safe because AuthorityFeatures has repr u32
-        Ok(unsafe { std::mem::transmute(<u32>::try_from(v)?) })
+        Ok(unsafe { std::mem::transmute::<u32, AuthorityFeatures>(v.try_into()?) })
     }
 }
 
-/// Details of a temporary authorization as provided by the /org/freedesktop/PolicyKit1/Authority object in the system bus.
+/// Details of a temporary authorization as provided by the /org/freedesktop/PolicyKit1/Authority
+/// object in the system bus.
 #[derive(Debug, Type, Deserialize)]
 pub struct TemporaryAuthorization {
     /// An opaque identifier for the temporary authorization.
@@ -75,8 +81,8 @@ pub struct TemporaryAuthorization {
     pub subject: Subject,
 
     /// When the temporary authorization was obtained, in seconds since the Epoch Jan 1, 1970 0:00
-    /// UTC. Note that the PolicyKit daemon is using monotonic time internally so the returned value
-    /// may change if system time changes.
+    /// UTC. Note that the PolicyKit daemon is using monotonic time internally so the returned
+    /// value may change if system time changes.
     pub time_obtained: u64,
 
     /// When the temporary authorization is set to expire, in seconds since the Epoch Jan 1, 1970
@@ -105,7 +111,7 @@ pub struct Identity<'a> {
 assert_impl_all!(Identity<'_>: Send, Sync, Unpin);
 
 fn pid_start_time(pid: u32) -> Result<u64, Error> {
-    let fname = format!("/proc/{}/stat", pid);
+    let fname = format!("/proc/{pid}/stat");
     let content = std::fs::read_to_string(fname)?;
 
     if let Some(i) = content.rfind(')') {
@@ -121,10 +127,10 @@ fn pid_start_time(pid: u32) -> Result<u64, Error> {
 // obsolete by the time this function returns; this function only guarantees that the UID was valid
 // at some point during its execution.
 fn pid_uid_racy(pid: u32) -> Result<u32, Error> {
-    let fname = format!("/proc/{}/status", pid);
+    let fname = format!("/proc/{pid}/status");
     let file = std::fs::File::open(fname)?;
     let lines = std::io::BufReader::new(file).lines();
-    for line in lines.flatten() {
+    for line in lines.map_while(Result::ok) {
         if line.starts_with("Uid:") {
             if let Some(uid) = line.split('\t').nth(1) {
                 return Ok(uid.parse()?);
@@ -141,13 +147,13 @@ fn pid_uid_racy(pid: u32) -> Result<u32, Error> {
 /// The following kinds of subjects are known:
 ///
 /// * Unix Process. `subject_kind` should be set to `unix-process` with keys `pid` (of type
-/// `uint32`) and `start-time` (of type `uint64`).
+///   `uint32`) and `start-time` (of type `uint64`).
 ///
 /// * Unix Session. `subject_kind` should be set to `unix-session` with the key `session-id` (of
-/// type `string`).
+///   type `string`).
 ///
 /// * System Bus Name. `subject_kind` should be set to `system-bus-name` with the key `name` (of
-/// type `string`).
+///   type `string`).
 #[derive(Debug, Type, Serialize, Deserialize)]
 pub struct Subject {
     /// The type of the subject.
@@ -170,7 +176,7 @@ impl Subject {
     /// * `start_time` - The start time for `pid` or `None` to look it up in e.g. `/proc`
     ///
     /// * `uid` - The (real, not effective) uid of the owner of `pid` or `None` to look it up in
-    /// e.g. `/proc`
+    ///   e.g. `/proc`
     pub fn new_for_owner(
         pid: u32,
         start_time: Option<u64>,
@@ -185,9 +191,9 @@ impl Subject {
             None => pid_uid_racy(pid)?,
         };
         let mut hashmap = HashMap::new();
-        hashmap.insert("pid".to_string(), Value::from(pid).into());
-        hashmap.insert("start-time".to_string(), Value::from(start_time).into());
-        hashmap.insert("uid".to_string(), Value::from(uid).into());
+        hashmap.insert("pid".to_string(), pid.into());
+        hashmap.insert("start-time".to_string(), start_time.into());
+        hashmap.insert("uid".to_string(), uid.into());
 
         Ok(Self {
             subject_kind: "unix-process".into(),
@@ -200,19 +206,22 @@ impl Subject {
     ///
     /// # Arguments
     ///
-    /// * `message_header` - The header of the message which caused an authentication to be necessary.
-    pub fn new_for_message_header(message_header: &zbus::MessageHeader<'_>) -> Result<Self, Error> {
+    /// * `message_header` - The header of the message which caused an authentication to be
+    ///   necessary.
+    pub fn new_for_message_header(
+        message_header: &zbus::message::Header<'_>,
+    ) -> Result<Self, Error> {
         let mut subject_details = HashMap::new();
         match message_header.sender() {
-            Ok(Some(sender)) => {
-                subject_details.insert("name".to_string(), Value::from(sender.clone()).into());
+            Some(sender) => {
+                subject_details.insert(
+                    "name".to_string(),
+                    OwnedUniqueName::from(sender.clone()).try_into().unwrap(),
+                );
             }
-            Ok(None) => {
+            None => {
                 return Err(Error::MissingSender);
             }
-            Err(e) => {
-                return Err(Error::BadSender(e));
-            }
         }
 
         Ok(Self {
@@ -285,12 +294,12 @@ assert_impl_all!(AuthorizationResult: Send, Sync, Unpin);
 
 /// This D-Bus interface is implemented by the /org/freedesktop/PolicyKit1/Authority object on the
 /// well-known name org.freedesktop.PolicyKit1 on the system message bus.
-#[dbus_proxy(
+#[zbus::proxy(
     interface = "org.freedesktop.PolicyKit1.Authority",
     default_service = "org.freedesktop.PolicyKit1",
     default_path = "/org/freedesktop/PolicyKit1/Authority"
 )]
-trait Authority {
+pub trait Authority {
     /// Method for authentication agents to invoke on successful authentication, intended only for
     /// use by a privileged helper process internal to polkit. This method will fail unless a
     /// sufficiently privileged +caller invokes it. Deprecated in favor of
@@ -349,7 +358,8 @@ trait Authority {
     /// The key `polkit.icon_name` is used to override the icon shown in the authentication dialog.
     ///
     /// If non-empty, then the request will fail with `org.freedesktop.PolicyKit1.Error.Failed`
-    /// unless the process doing the check itself is sufficiently authorized (e.g. running as uid 0).
+    /// unless the process doing the check itself is sufficiently authorized (e.g. running as uid
+    /// 0).
     ///
     /// * `flags` - A set of `CheckAuthorizationFlags`.
     ///
@@ -408,7 +418,7 @@ trait Authority {
         subject: &Subject,
         locale: &str,
         object_path: &str,
-        options: &std::collections::HashMap<&str, zvariant::Value<'_>>,
+        options: &std::collections::HashMap<&str, Value<'_>>,
     ) -> zbus::Result<()>;
 
     /// Revokes all temporary authorizations that applies to `id`.
@@ -431,21 +441,22 @@ trait Authority {
     ) -> zbus::Result<()>;
 
     /// This signal is emitted when actions and/or authorizations change
-    #[dbus_proxy(signal)]
+    #[zbus(signal)]
     fn changed(&self) -> fdo::Result<()>;
 
     /// The features supported by the currently used Authority backend.
-    #[dbus_proxy(property)]
+    #[zbus(property)]
     fn backend_features(&self) -> fdo::Result<AuthorityFeatures>;
 
     /// The name of the currently used Authority backend.
-    #[dbus_proxy(property)]
+    #[zbus(property)]
     fn backend_name(&self) -> fdo::Result<String>;
 
     /// The version of the currently used Authority backend.
-    #[dbus_proxy(property)]
+    #[zbus(property)]
     fn backend_version(&self) -> fdo::Result<String>;
 }
 
 assert_impl_all!(AuthorityProxy<'_>: Send, Sync, Unpin);
+#[cfg(feature = "blocking-api")]
 assert_impl_all!(AuthorityProxyBlocking<'_>: Send, Sync, Unpin);
